// *************************************************************************************
// SCICHART® Copyright SciChart Ltd. 2011-2023. All rights reserved.
//  
// Web: http://www.scichart.com
//   Support: support@scichart.com
//   Sales:   sales@scichart.com
// 
// SeriesTooltips3DChart.xaml.cs is part of the SCICHART® Examples. Permission is hereby granted
// to modify, create derivative works, distribute and publish any part of this source
// code whether for commercial, private or personal use. 
// 
// The SCICHART® examples are distributed in the hope that they will be useful, but
// without any warranty. It is provided "AS IS" without warranty of any kind, either
// expressed or implied. 
// *************************************************************************************
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Media3D;
using SciChart.Charting3D.Model;
using SciChart.Charting3D.Model.ChartData;

namespace SciChart.Examples.Examples.Charts3D.TooltipsAndHitTest3DCharts
{
    /// <summary>
    /// Interaction logic for SeriesTooltips3DChart.xaml
    /// </summary>
    public partial class SeriesTooltips3DChart : UserControl
    {
        private const int SegmentsCount = 25;
        private const double YAngle = -65;

        private readonly Color _mailColor = Color.FromArgb(0xFF, 0x64, 0xBA, 0xE4);
        private readonly Color _secondaryColor = Color.FromArgb(0xFF, 0xDC, 0x79, 0x69);
        private readonly double _rotationAngle;

        public SeriesTooltips3DChart()
        {
            InitializeComponent();

            // Provide TooltipLabelDataContextSelector to get access to the hit-test info
            TooltipModifier3D.TooltipLabelDataContextSelector = seriesInfo3D =>
            {
                ChartPositionTextBlock.Text = $"Chart Coordinates: X: {seriesInfo3D.FormattedXValue}, Y: {seriesInfo3D.FormattedYValue}, Z: {seriesInfo3D.FormattedZValue}";

                var worldX = SciChart.XAxis.GetCoordinate(seriesInfo3D.XValue);
                var worldY = SciChart.XAxis.GetCoordinate(seriesInfo3D.XValue);
                var worldZ = SciChart.XAxis.GetCoordinate(seriesInfo3D.XValue);
                ScreenPositionTextBlock.Text = $"World Coordinates: X: {worldX:N2}, Y: {worldY:N2}, Z: {worldZ:N2}";

                // Start an animation to hide the info labels
                var animation = new DoubleAnimation(1, 0, TimeSpan.FromMilliseconds(1000));
                ChartPositionTextBlock.BeginAnimation(OpacityProperty, animation, HandoffBehavior.SnapshotAndReplace);
                ScreenPositionTextBlock.BeginAnimation(OpacityProperty, animation, HandoffBehavior.SnapshotAndReplace);

                return seriesInfo3D;
            };

            _rotationAngle = 360d / 45;
            
            Loaded += (sender, args) => Initialize();
        }

        private void Initialize()
        {
            var xyzDataSeries3D = new XyzDataSeries3D<double>();

            var currentAngle = 0d;
            for (var i = -SegmentsCount; i < SegmentsCount + 1; i++)
            {
                AddSegment(xyzDataSeries3D, i, currentAngle);
                currentAngle = (currentAngle + _rotationAngle) % 360;
            }

            SciChart.RenderableSeries[0].DataSeries = xyzDataSeries3D;
        }

        private void AddSegment(IXyzDataSeries3D<double, double, double> xyzDataSeries3D, double y, double currentAngle)
        {
            var temp = RotatePoint(new Point(-4, 0), DegreeToRadian(currentAngle));
            var point = RotateAroundZ(new Point3D(temp.X, y, temp.Y), DegreeToRadian(YAngle));
            xyzDataSeries3D.Append(point.X, point.Y, point.Z, new PointMetadata3D(_mailColor));

            temp = RotatePoint(new Point(4, 0), DegreeToRadian(currentAngle));
            point = RotateAroundZ(new Point3D(temp.X, y, temp.Y), DegreeToRadian(YAngle));
            xyzDataSeries3D.Append(point.X, point.Y, point.Z, new PointMetadata3D(_secondaryColor));
        }

        private Point RotatePoint(Point point, double angle)
        {
            var x = point.X * Math.Cos(angle) - point.Y * Math.Sin(angle);
            var y = point.X * Math.Sin(angle) + point.Y * Math.Cos(angle);

            return new Point(x, y);
        }

        private Point3D RotateAroundZ(Point3D point, double angle)
        {
            var x = point.X * Math.Cos(angle) - point.Y * Math.Sin(angle);
            var y = point.X * Math.Sin(angle) + point.Y * Math.Cos(angle);

            return new Point3D(x, y, point.Z);
        }

        private double DegreeToRadian(double angle)
        {
            return Math.PI * angle / 180.0;
        }
    }
}